@@ -478,6 +478,12 @@ arv_device_status_get_type (void)
 			{ ARV_DEVICE_STATUS_WRITE_ERROR,
 			  "ARV_DEVICE_STATUS_WRITE_ERROR",
 			  "write-error" },
+			{ ARV_DEVICE_STATUS_TRANSFER_ERROR,
+			  "ARV_DEVICE_STATUS_TRANSFER_ERROR",
+			  "transfer-error" },
+			{ ARV_DEVICE_STATUS_NOT_CONNECTED,
+			  "ARV_DEVICE_STATUS_NOT_CONNECTED",
+			  "not-connected" },
 			{ 0, NULL, NULL }
 		};
 		the_type = g_enum_register_static (
