@@ -29,11 +29,19 @@
 
 G_BEGIN_DECLS
 
-#define	ARV_GV_DEVICE_GVCP_N_RETRIES_DEFAULT	5
-#define	ARV_GV_DEVICE_GVCP_TIMEOUT_MS_DEFAULT	500
-#define	ARV_GV_DEVICE_HEARTBEAT_PERIOD_US	1000000
-#define	ARV_GV_DEVICE_HEARTBEAT_RETRY_DELAY_US	10000
-#define	ARV_GV_DEVICE_HEARTBEAT_RETRY_TIMEOUT_S	5.0		/* FIXME */
+#ifdef ARAVIS_FAST_HEARTBEAT
+    #define ARV_GV_DEVICE_GVCP_N_RETRIES_DEFAULT    3
+    #define ARV_GV_DEVICE_GVCP_TIMEOUT_MS_DEFAULT   25
+    #define ARV_GV_DEVICE_HEARTBEAT_PERIOD_US       50000
+    #define ARV_GV_DEVICE_HEARTBEAT_RETRY_DELAY_US  1000
+    #define ARV_GV_DEVICE_HEARTBEAT_RETRY_TIMEOUT_S 0.25
+#else
+    #define ARV_GV_DEVICE_GVCP_N_RETRIES_DEFAULT    5
+    #define ARV_GV_DEVICE_GVCP_TIMEOUT_MS_DEFAULT   500
+    #define ARV_GV_DEVICE_HEARTBEAT_PERIOD_US       1000000
+    #define ARV_GV_DEVICE_HEARTBEAT_RETRY_DELAY_US  10000
+    #define ARV_GV_DEVICE_HEARTBEAT_RETRY_TIMEOUT_S 5.0		/* FIXME */
+#endif
 
 #define ARV_GV_DEVICE_GVSP_PACKET_SIZE_DEFAULT	1500
 
