@@ -60,6 +60,31 @@ typedef enum {
 	ARV_GVSP_CONTENT_TYPE_DATA_BLOCK =	0x03
 } ArvGvspContentType;
 
+/**
+ * ArvGvspPayloadType:
+ * @ARV_GVSP_PAYLOAD_TYPE_IMAGE: image data
+ * @ARV_GVSP_PAYLOAD_TYPE_RAWDATA: raw data
+ * @ARV_GVSP_PAYLOAD_TYPE_FILE: file
+ * @ARV_GVSP_PAYLOAD_TYPE_CHUNK_DATA: chunk data
+ * @ARV_GVSP_PAYLOAD_TYPE_EXTENDED_CHUNK_DATA: extended chunk data
+ * @ARV_GVSP_PAYLOAD_TYPE_JPEG: JPEG data
+ * @ARV_GVSP_PAYLOAD_TYPE_JPEG2000: JPEG2000 data
+ * @ARV_GVSP_PAYLOAD_TYPE_H264: h264 data
+ * @ARV_GVSP_PAYLOAD_TYPE_MULTIZONE_IMAGE: multizone image
+*/
+
+typedef enum {
+	ARV_GVSP_PAYLOAD_TYPE_IMAGE =			0x0001,
+	ARV_GVSP_PAYLOAD_TYPE_RAWDATA = 		0x0002,
+	ARV_GVSP_PAYLOAD_TYPE_FILE = 			0x0003,
+	ARV_GVSP_PAYLOAD_TYPE_CHUNK_DATA = 		0x0004,
+	ARV_GVSP_PAYLOAD_TYPE_EXTENDED_CHUNK_DATA = 	0x0005, /* Deprecated */
+	ARV_GVSP_PAYLOAD_TYPE_JPEG = 			0x0006,
+	ARV_GVSP_PAYLOAD_TYPE_JPEG2000 = 		0x0007,
+	ARV_GVSP_PAYLOAD_TYPE_H264 = 			0x0008,
+	ARV_GVSP_PAYLOAD_TYPE_MULTIZONE_IMAGE = 	0x0009
+} ArvGvspPayloadType;
+
 #define ARAVIS_PACKED_STRUCTURE __attribute__((__packed__))
 
 /**
@@ -115,7 +140,6 @@ typedef struct ARAVIS_PACKED_STRUCTURE {
 	guint32 data1;
 } ArvGvspDataTrailer;
 
-#undef ARAVIS_PACKED_STRUCTURE
 
 /**
  * ArvGvspPacket:
@@ -125,11 +149,13 @@ typedef struct ARAVIS_PACKED_STRUCTURE {
  * GVSP packet structure.
  */
 
-typedef struct {
+typedef struct ARAVIS_PACKED_STRUCTURE {
 	ArvGvspHeader header;
 	guint8 data[];
 } ArvGvspPacket;
 
+#undef ARAVIS_PACKED_STRUCTURE
+
 ArvGvspPacket *		arv_gvsp_packet_new_data_leader		(guint16 frame_id, guint32 packet_id,
 								 guint64 timestamp, ArvPixelFormat pixel_format,
 								 guint32 width, guint32 height,
@@ -168,6 +194,15 @@ arv_gvsp_packet_get_frame_id (const ArvGvspPacket *packet)
 	return g_ntohs (packet->header.frame_id);
 }
 
+static inline ArvGvspPayloadType
+arv_gvsp_packet_get_payload_type (const ArvGvspPacket *packet)
+{
+	ArvGvspDataLeader *leader;
+
+	leader = (ArvGvspDataLeader *) &packet->data;
+	return (ArvGvspPayloadType) g_ntohl (leader->payload_type);
+}
+
 static inline guint32
 arv_gvsp_packet_get_x_offset (const ArvGvspPacket *packet)
 {
