@@ -30,6 +30,83 @@
 #include <memory.h>
 #include <libnotify/notify.h>
 
+static gboolean has_autovideo_sink = FALSE;
+static gboolean has_gtksink = FALSE;
+static gboolean has_gtkglsink = FALSE;
+
+static gboolean
+gstreamer_plugin_check (void)
+{
+	static gsize check_done = 0;
+	static gboolean check_success = FALSE;
+
+	if (g_once_init_enter (&check_done)) {
+		GstRegistry *registry;
+		GstPluginFeature *feature;
+		unsigned int i;
+		gboolean success = TRUE;
+
+		static char *plugins[] = {
+			"appsrc",
+			"videoconvert",
+			"videoflip",
+			"bayer2rgb"
+		};
+
+		registry = gst_registry_get ();
+
+		for (i = 0; i < G_N_ELEMENTS (plugins); i++) {
+			feature = gst_registry_lookup_feature (registry, plugins[i]);
+			if (!GST_IS_PLUGIN_FEATURE (feature)) {
+				g_print ("Gstreamer plugin '%s' is missing.\n", plugins[i]);
+				success = FALSE;
+			}
+			else
+
+				g_object_unref (feature);
+		}
+
+		feature = gst_registry_lookup_feature (registry, "autovideosink");
+		if (GST_IS_PLUGIN_FEATURE (feature)) {
+			has_autovideo_sink = TRUE;
+			g_object_unref (feature);
+		}
+
+		feature = gst_registry_lookup_feature (registry, "gtksink");
+		if (GST_IS_PLUGIN_FEATURE (feature)) {
+			has_gtksink = TRUE;
+			g_object_unref (feature);
+		}
+
+		feature = gst_registry_lookup_feature (registry, "gtkglsink");
+		if (GST_IS_PLUGIN_FEATURE (feature)) {
+			has_gtkglsink = TRUE;
+			g_object_unref (feature);
+		}
+
+		if (!has_autovideo_sink && !has_gtkglsink && !has_gtksink) {
+			g_print ("Missing GStreamer video output plugin (autovideosink, gtksink or gtkglsink)\n");
+			success = FALSE;
+		}
+
+		if (!success)
+			g_print ("Check your gstreamer installation.\n");
+
+		/* Kludge, prevent autoloading of coglsink, which doesn't seem to work for us */
+		feature = gst_registry_lookup_feature (registry, "coglsink");
+		if (GST_IS_PLUGIN_FEATURE (feature)) {
+			gst_plugin_feature_set_rank (feature, GST_RANK_NONE);
+			g_object_unref (feature);
+		}
+
+		check_success = success;
+
+		g_once_init_leave (&check_done, 1);
+	}
+
+	return check_success;
+}
+
 typedef struct {
 	GtkApplication parent_instance;
 
@@ -50,7 +127,7 @@ typedef struct {
 	gboolean flip_horizontal;
 
 	double exposure_min, exposure_max;
-	
+
 	NotifyNotification *notification;
 
 	GtkWidget *main_window;
@@ -84,6 +161,7 @@ typedef struct {
 	GtkWidget *auto_gain_toggle;
 	GtkWidget *acquisition_button;
 
+	gulong camera_selected;
 	gulong exposure_spin_changed;
 	gulong gain_spin_changed;
 	gulong exposure_hscale_changed;
@@ -105,6 +183,8 @@ typedef struct {
 	gboolean packet_resend;
 	guint packet_timeout;
 	guint frame_retention;
+
+	gulong video_window_xid;
 } ArvViewer;
 
 typedef GtkApplicationClass ArvViewerClass;
@@ -160,7 +240,7 @@ arv_viewer_value_from_log (double value, double min, double max)
 }
 
 static GstBuffer *
-arv_to_gst_buffer (ArvBuffer *arv_buffer) 
+arv_to_gst_buffer (ArvBuffer *arv_buffer)
 {
 	GstBuffer *buffer;
 	int arv_row_stride;
@@ -182,7 +262,7 @@ arv_to_gst_buffer (ArvBuffer *arv_buffer)
 		gst_row_stride = (arv_row_stride & ~(0x3)) + 4;
 
 		size = height * gst_row_stride;
-		data = g_malloc (size);	
+		data = g_malloc (size);
 
 		for (i = 0; i < height; i++)
 			memcpy (((char *) data) + i * gst_row_stride, buffer_data + i * arv_row_stride, arv_row_stride);
@@ -504,7 +584,7 @@ stream_cb (void *user_data, ArvStreamCallbackType type, ArvBuffer *buffer)
 	}
 }
 
-static void 
+static void
 update_camera_region (ArvViewer *viewer)
 {
 	gint x, y, width, height;
@@ -559,17 +639,25 @@ camera_region_cb (GtkSpinButton *spin_button, ArvViewer *viewer)
 {
 	int x = gtk_spin_button_get_value (GTK_SPIN_BUTTON (viewer->camera_x));
 	int y = gtk_spin_button_get_value (GTK_SPIN_BUTTON (viewer->camera_y));
-	int dx = gtk_spin_button_get_value (GTK_SPIN_BUTTON (viewer->camera_binning_x));
-	int dy = gtk_spin_button_get_value (GTK_SPIN_BUTTON (viewer->camera_binning_y));
 	int width = gtk_spin_button_get_value (GTK_SPIN_BUTTON (viewer->camera_width));
 	int height = gtk_spin_button_get_value (GTK_SPIN_BUTTON (viewer->camera_height));
 
-	arv_camera_set_binning (viewer->camera, dx, dy);
 	arv_camera_set_region (viewer->camera, x, y, width, height);
 
 	update_camera_region (viewer);
 }
 
+void
+camera_binning_cb (GtkSpinButton *spin_button, ArvViewer *viewer)
+{
+	int dx = gtk_spin_button_get_value (GTK_SPIN_BUTTON (viewer->camera_binning_x));
+	int dy = gtk_spin_button_get_value (GTK_SPIN_BUTTON (viewer->camera_binning_y));
+
+	arv_camera_set_binning (viewer->camera, dx, dy);
+
+	update_camera_region (viewer);
+}
+
 void
 pixel_format_combo_cb (GtkComboBoxText *combo, ArvViewer *viewer)
 {
@@ -588,6 +676,10 @@ update_device_list_cb (GtkToolButton *button, ArvViewer *viewer)
 	unsigned int n_devices;
 	unsigned int i;
 
+	gtk_widget_set_sensitive (viewer->video_mode_button, FALSE);
+	gtk_widget_set_sensitive (viewer->camera_parameters, FALSE);
+
+	g_signal_handler_block (gtk_tree_view_get_selection (GTK_TREE_VIEW (viewer->camera_tree)), viewer->camera_selected);
 	list_store = GTK_LIST_STORE (gtk_tree_view_get_model (GTK_TREE_VIEW (viewer->camera_tree)));
 	gtk_list_store_clear (list_store);
 	arv_update_device_list ();
@@ -601,6 +693,7 @@ update_device_list_cb (GtkToolButton *button, ArvViewer *viewer)
 				    3, arv_get_device_serial_nbr (i),
 				    -1);
 	}
+	g_signal_handler_unblock (gtk_tree_view_get_selection (GTK_TREE_VIEW (viewer->camera_tree)), viewer->camera_selected);
 }
 
 static void
@@ -626,14 +719,34 @@ stop_video (ArvViewer *viewer)
 	gtk_container_foreach (GTK_CONTAINER (viewer->video_frame), remove_widget, viewer->video_frame);
 }
 
+static GstBusSyncReply
+bus_sync_handler (GstBus *bus, GstMessage *message, gpointer user_data)
+{
+	ArvViewer *viewer = user_data;
+
+	if (!gst_is_video_overlay_prepare_window_handle_message(message))
+		return GST_BUS_PASS;
+
+	if (viewer->video_window_xid != 0) {
+		GstVideoOverlay *videooverlay;
+
+		videooverlay = GST_VIDEO_OVERLAY (GST_MESSAGE_SRC (message));
+		gst_video_overlay_set_window_handle (videooverlay, viewer->video_window_xid);
+	} else {
+		g_warning ("Should have obtained video_window_xid by now!");
+	}
+
+	gst_message_unref (message);
+
+	return GST_BUS_DROP;
+}
+
 static gboolean
 start_video (ArvViewer *viewer)
 {
 	GstElement *videoconvert;
-	GstElement *gl_upload;
 	GstElement *videosink;
 	GstCaps *caps;
-	GtkWidget *video_widget;
 	ArvPixelFormat pixel_format;
 	double frame_rate;
 	double gain_min, gain_max;
@@ -739,7 +852,7 @@ start_video (ArvViewer *viewer)
 
 	g_signal_handler_block (viewer->auto_gain_toggle, viewer->auto_gain_clicked);
 	g_signal_handler_block (viewer->auto_exposure_toggle, viewer->auto_exposure_clicked);
-	
+
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (viewer->auto_gain_toggle), auto_gain);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (viewer->auto_exposure_toggle), auto_exposure);
 
@@ -763,27 +876,44 @@ start_video (ArvViewer *viewer)
 	viewer->appsrc = gst_element_factory_make ("appsrc", NULL);
 	videoconvert = gst_element_factory_make ("videoconvert", NULL);
 	viewer->transform = gst_element_factory_make ("videoflip", NULL);
-	gl_upload = gst_element_factory_make ("glupload", NULL);
-	videosink = gst_element_factory_make ("gtkglsink", NULL);
 
-	g_object_get (videosink, "widget", &video_widget, NULL);
-	gtk_container_add (GTK_CONTAINER (viewer->video_frame), video_widget);
-	gtk_widget_show (video_widget);
+	gst_bin_add_many (GST_BIN (viewer->pipeline), viewer->appsrc, videoconvert, viewer->transform, NULL);
 
 	if (g_str_has_prefix (caps_string, "video/x-bayer")) {
 		GstElement *bayer2rgb;
 
 		bayer2rgb = gst_element_factory_make ("bayer2rgb", NULL);
-
-		gst_bin_add_many (GST_BIN (viewer->pipeline), viewer->appsrc, bayer2rgb,
-				  videoconvert, viewer->transform, gl_upload, videosink, NULL);
-		gst_element_link_many (viewer->appsrc, bayer2rgb,
-				       videoconvert, viewer->transform, gl_upload, videosink, NULL);
+		gst_bin_add (GST_BIN (viewer->pipeline), bayer2rgb);
+		gst_element_link_many (viewer->appsrc, bayer2rgb, videoconvert, viewer->transform, NULL);
 	} else {
-		gst_bin_add_many (GST_BIN (viewer->pipeline), viewer->appsrc,
-				  videoconvert, viewer->transform, gl_upload, videosink, NULL);
-		gst_element_link_many (viewer->appsrc, videoconvert,
-				       viewer->transform, gl_upload, videosink, NULL);
+		gst_element_link_many (viewer->appsrc, videoconvert, viewer->transform, NULL);
+	}
+
+	if (has_gtksink || has_gtkglsink) {
+		GtkWidget *video_widget;
+
+		videosink = gst_element_factory_make ("gtkglsink", NULL);
+
+		if (GST_IS_ELEMENT (videosink)) {
+			GstElement *glupload;
+
+			glupload = gst_element_factory_make ("glupload", NULL);
+			gst_bin_add_many (GST_BIN (viewer->pipeline), glupload, videosink, NULL);
+			gst_element_link_many (viewer->transform, glupload, videosink, NULL);
+		} else {
+			videosink = gst_element_factory_make ("gtksink", NULL);
+			gst_element_link_many (viewer->transform, videosink, NULL);
+		}
+
+		g_object_get (videosink, "widget", &video_widget, NULL);
+		gtk_container_add (GTK_CONTAINER (viewer->video_frame), video_widget);
+		gtk_widget_show (video_widget);
+		g_object_set(G_OBJECT (video_widget), "force-aspect-ratio", TRUE, NULL);
+		gtk_widget_set_size_request (video_widget, 640, 480);
+	} else {
+		videosink = gst_element_factory_make ("autovideosink", NULL);
+		gst_bin_add (GST_BIN (viewer->pipeline), videosink);
+		gst_element_link_many (viewer->transform, videosink, NULL);
 	}
 
 	caps = gst_caps_from_string (caps_string);
@@ -796,25 +926,38 @@ start_video (ArvViewer *viewer)
 	gst_caps_unref (caps);
 
 	g_object_set(G_OBJECT (viewer->appsrc), "format", GST_FORMAT_TIME, NULL);
-	g_object_set(G_OBJECT (video_widget), "force-aspect-ratio", TRUE, NULL);	
+
+	if (!has_gtkglsink && !has_gtksink) {
+		GstBus *bus;
+
+		bus = gst_pipeline_get_bus (GST_PIPELINE (viewer->pipeline));
+		gst_bus_set_sync_handler (bus, (GstBusSyncHandler) bus_sync_handler, viewer, NULL);
+		gst_object_unref (bus);
+	}
 
 	gst_element_set_state (viewer->pipeline, GST_STATE_PLAYING);
 
 	g_signal_connect (viewer->stream, "new-buffer", G_CALLBACK (new_buffer_cb), viewer);
 
-	gtk_widget_set_size_request (video_widget, 640, 480);
-
 	return TRUE;
 }
 
-/* FIXME */
-#if 0
+static gboolean
+select_camera_list_mode (gpointer user_data)
+{
+	ArvViewer *viewer = user_data;
+
+	select_mode (viewer, ARV_VIEWER_MODE_CAMERA_LIST);
+	update_device_list_cb (GTK_TOOL_BUTTON (viewer->refresh_button), viewer);
+
+	return FALSE;
+}
+
 static void
 control_lost_cb (ArvCamera *camera, ArvViewer *viewer)
 {
-	select_mode (viewer, ARV_VIEWER_MODE_CAMERA_LIST);
+	g_main_context_invoke (NULL, select_camera_list_mode, viewer);
 }
-#endif
 
 static void
 stop_camera (ArvViewer *viewer)
@@ -831,9 +974,10 @@ start_camera (ArvViewer *viewer, const char *camera_id)
 {
 	GtkTreeIter iter;
 	GtkListStore *list_store;
-	const char *pixel_format;
-	const char **pixel_formats;
-	guint i, n_pixel_formats;
+	gint64 *pixel_formats;
+	const char *pixel_format_string;
+	const char **pixel_format_strings;
+	guint i, n_pixel_formats, n_pixel_format_strings, n_valid_formats;
 	gboolean binning_available;
 
 	stop_camera (viewer);
@@ -844,7 +988,7 @@ start_camera (ArvViewer *viewer, const char *camera_id)
 		return FALSE;
 
 	viewer->camera_name = g_strdup (camera_id);
-	
+
 	gtk_widget_set_sensitive (viewer->camera_parameters, TRUE);
 
 	arv_camera_set_chunk_mode (viewer->camera, FALSE);
@@ -855,16 +999,23 @@ start_camera (ArvViewer *viewer, const char *camera_id)
 
 	list_store = GTK_LIST_STORE (gtk_combo_box_get_model (GTK_COMBO_BOX (viewer->pixel_format_combo)));
 	gtk_list_store_clear (list_store);
-
-	pixel_formats = arv_camera_get_available_pixel_formats_as_strings (viewer->camera, &n_pixel_formats);
-	pixel_format = arv_camera_get_pixel_format_as_string (viewer->camera);
+	n_valid_formats = 0;
+	pixel_format_strings = arv_camera_get_available_pixel_formats_as_strings (viewer->camera, &n_pixel_format_strings);
+	pixel_formats = arv_camera_get_available_pixel_formats (viewer->camera, &n_pixel_formats);
+	g_assert (n_pixel_formats == n_pixel_format_strings);
+	pixel_format_string = arv_camera_get_pixel_format_as_string (viewer->camera);
 	for (i = 0; i < n_pixel_formats; i++) {
-		gtk_list_store_append (list_store, &iter);
-		gtk_list_store_set (list_store, &iter, 0, pixel_formats[i], -1);
-		if (g_strcmp0 (pixel_formats[i], pixel_format) == 0)
-			gtk_combo_box_set_active (GTK_COMBO_BOX (viewer->pixel_format_combo), i);
+		if (arv_pixel_format_to_gst_caps_string (pixel_formats[i]) != NULL) {
+			gtk_list_store_append (list_store, &iter);
+			gtk_list_store_set (list_store, &iter, 0, pixel_format_strings[i], -1);
+			if (g_strcmp0 (pixel_format_strings[i], pixel_format_string) == 0)
+				gtk_combo_box_set_active (GTK_COMBO_BOX (viewer->pixel_format_combo), n_valid_formats);
+			n_valid_formats++;
+		}
 	}
-	gtk_widget_set_sensitive (viewer->pixel_format_combo, n_pixel_formats > 1);
+	g_free (pixel_formats);
+	g_free (pixel_format_strings);
+	gtk_widget_set_sensitive (viewer->pixel_format_combo, n_valid_formats > 1);
 	gtk_widget_set_sensitive (viewer->video_mode_button, TRUE);
 
 	binning_available = arv_camera_is_binning_available (viewer->camera);
@@ -873,10 +1024,7 @@ start_camera (ArvViewer *viewer, const char *camera_id)
 
 	g_signal_handler_unblock (viewer->pixel_format_combo, viewer->pixel_format_changed);
 
-	/* FIXME */
-#if 0
 	g_signal_connect (arv_camera_get_device (viewer->camera), "control-lost", G_CALLBACK (control_lost_cb), viewer);
-#endif
 
 	return TRUE;
 }
@@ -960,19 +1108,28 @@ arv_viewer_quit_cb (GtkApplicationWindow *window, ArvViewer *viewer)
 	g_application_quit (G_APPLICATION (viewer));
 }
 
+static void
+video_frame_realize_cb (GtkWidget * widget, ArvViewer *viewer)
+{
+	viewer->video_window_xid = GDK_WINDOW_XID (gtk_widget_get_window (widget));
+}
+
 static void
 activate (GApplication *application)
 {
 	ArvViewer *viewer = (ArvViewer *) application;
 	GtkBuilder *builder;
 	char *ui_filename;
+	GError *err = NULL;
 
 	builder = gtk_builder_new ();
 
 	ui_filename = g_build_filename (ARAVIS_DATA_DIR, "arv-viewer.ui", NULL);
 
-	if (!gtk_builder_add_from_file (builder, ui_filename, NULL))
-		g_error ("The user interface file is missing ('%s')", ui_filename);
+	if (!gtk_builder_add_from_file (builder, ui_filename, &err)) {
+		g_error ("Cant't load user interface file: %s", err->message);
+		g_error_free (err);
+	}
 
 	g_free (ui_filename);
 
@@ -1018,8 +1175,6 @@ activate (GApplication *application)
 	g_signal_connect (viewer->refresh_button, "clicked", G_CALLBACK (update_device_list_cb), viewer);
 	g_signal_connect (viewer->video_mode_button, "clicked", G_CALLBACK (switch_to_video_mode_cb), viewer);
 	g_signal_connect (viewer->back_button, "clicked", G_CALLBACK (switch_to_camera_list_cb), viewer);
-	g_signal_connect (gtk_tree_view_get_selection (GTK_TREE_VIEW (viewer->camera_tree)), "changed",
-			  G_CALLBACK (camera_selection_changed_cb), viewer);
 	g_signal_connect (viewer->main_window, "destroy", G_CALLBACK (arv_viewer_quit_cb), viewer);
 	g_signal_connect (viewer->snapshot_button, "clicked", G_CALLBACK (snapshot_cb), viewer);
 	g_signal_connect (viewer->rotate_cw_button, "clicked", G_CALLBACK (rotate_cw_cb), viewer);
@@ -1028,6 +1183,12 @@ activate (GApplication *application)
 	g_signal_connect (viewer->frame_rate_entry, "activate", G_CALLBACK (frame_rate_entry_cb), viewer);
 	g_signal_connect (viewer->frame_rate_entry, "focus-out-event", G_CALLBACK (frame_rate_entry_focus_cb), viewer);
 
+	if (!has_gtksink && !has_gtkglsink) {
+		g_signal_connect (viewer->video_frame, "realize", G_CALLBACK (video_frame_realize_cb), viewer);
+	}
+
+	viewer->camera_selected = g_signal_connect (gtk_tree_view_get_selection (GTK_TREE_VIEW (viewer->camera_tree)), "changed",
+						    G_CALLBACK (camera_selection_changed_cb), viewer);
 	viewer->exposure_spin_changed = g_signal_connect (viewer->exposure_spin_button, "value-changed",
 							  G_CALLBACK (exposure_spin_cb), viewer);
 	viewer->gain_spin_changed = g_signal_connect (viewer->gain_spin_button, "value-changed",
@@ -1040,20 +1201,20 @@ activate (GApplication *application)
 							  G_CALLBACK (auto_exposure_cb), viewer);
 	viewer->auto_gain_clicked = g_signal_connect (viewer->auto_gain_toggle, "clicked",
 						      G_CALLBACK (auto_gain_cb), viewer);
+	viewer->pixel_format_changed = g_signal_connect (viewer->pixel_format_combo, "changed",
+							 G_CALLBACK (pixel_format_combo_cb), viewer);
 	viewer->camera_x_changed = g_signal_connect (viewer->camera_x, "value-changed",
 						     G_CALLBACK (camera_region_cb), viewer);
 	viewer->camera_y_changed = g_signal_connect (viewer->camera_y, "value-changed",
 						     G_CALLBACK (camera_region_cb), viewer);
-	viewer->camera_binning_x_changed = g_signal_connect (viewer->camera_binning_x, "value-changed",
-							     G_CALLBACK (camera_region_cb), viewer);
-	viewer->camera_binning_y_changed = g_signal_connect (viewer->camera_binning_y, "value-changed",
-							     G_CALLBACK (camera_region_cb), viewer);
 	viewer->camera_width_changed = g_signal_connect (viewer->camera_width, "value-changed",
 							 G_CALLBACK (camera_region_cb), viewer);
 	viewer->camera_height_changed = g_signal_connect (viewer->camera_height, "value-changed",
 							  G_CALLBACK (camera_region_cb), viewer);
-	viewer->pixel_format_changed = g_signal_connect (viewer->pixel_format_combo, "changed",
-							 G_CALLBACK (pixel_format_combo_cb), viewer);
+	viewer->camera_binning_x_changed = g_signal_connect (viewer->camera_binning_x, "value-changed",
+							     G_CALLBACK (camera_binning_cb), viewer);
+	viewer->camera_binning_y_changed = g_signal_connect (viewer->camera_binning_y, "value-changed",
+							     G_CALLBACK (camera_binning_cb), viewer);
 
 	gtk_widget_set_sensitive (viewer->camera_parameters, FALSE);
 	select_mode (viewer, ARV_VIEWER_MODE_CAMERA_LIST);
@@ -1091,13 +1252,15 @@ arv_viewer_new (void)
 {
   ArvViewer *arv_viewer;
 
+  if (!gstreamer_plugin_check ())
+	  return NULL;
+
   g_set_application_name ("ArvViewer");
 
   arv_viewer = g_object_new (arv_viewer_get_type (),
 			     "application-id", "org.aravis.ArvViewer",
-			     "flags", G_APPLICATION_FLAGS_NONE,
+			     "flags", G_APPLICATION_NON_UNIQUE,
 			     "inactivity-timeout", 30000,
-/*                             "register-session", TRUE,*/
 			     NULL);
 
   return arv_viewer;
